@@ -33,6 +33,8 @@
 #include "mime-sql.h"
 #include "mime-programs-sql.h"
 
+#include "bluetooth.h"
+
 #define _(x) dgettext(PACKAGE, x)
 
 #define COMPLETED 0
@@ -43,10 +45,13 @@
 #define DEFAULT_ICON_PATH "/pixmaps"
 #define ZOOM_INCREMENT 8
 
+#define MAX_SYM_DEPTH 256
+
 GtkWidget *window;
 GtkWidget *combo;
 GtkWidget *view_scrolld;
 GtkWidget *view_widget;
+GtkWidget *bluetooth_menu_item;
 
 GdkPixbuf *default_pixbuf;
 
@@ -76,9 +81,9 @@ typedef struct
 {
   gchar *filename;
   GnomeVFSFileInfo *vfs;
-} FileInfomation;
+} FileInformation;
 
-FileInfomation *current_popup_file;
+FileInformation *current_popup_file;
 
 struct gpe_icon my_icons[] = {
   { "left", "left" },
@@ -108,11 +113,14 @@ static void popup_ask_move_file (void);
 static void popup_ask_rename_file (void);
 static void popup_ask_delete_file (void);
 static void show_file_properties (void);
+static void refresh_current_directory (void);
+static void send_with_bluetooth (void);
 
 static GtkItemFactoryEntry menu_items[] =
 {
   { "/Open Wit_h",	 NULL, popup_ask_open_with,  0, "<StockItem>", GTK_STOCK_OPEN },
   { "/sep1",	         NULL, NULL,	             0, "<Separator>" },
+  { "/Send via Bluetooth", NULL, send_with_bluetooth, 0, "<Item>" },
   { "/_Move",            NULL, popup_ask_move_file,            0, "<Item>" },
   { "/_Rename",          NULL, popup_ask_rename_file,          0, "<Item>" },
   { "/_Delete",          NULL, popup_ask_delete_file,         0, "<StockItem>", GTK_STOCK_DELETE },
@@ -230,10 +238,10 @@ static void
 open_with (GtkButton *button, gpointer data)
 {
   GnomeVFSMimeApplication *application;
-  FileInfomation *file_info;
+  FileInformation *file_info;
   char *command;
 
-  file_info = gtk_object_get_data (GTK_OBJECT (button), "FileInfomation");
+  file_info = gtk_object_get_data (GTK_OBJECT (button), "FileInformation");
   application = gtk_object_get_data (GTK_OBJECT (data), "GnomeVFSMimeApplication");
 
   if (application->requires_terminal)
@@ -255,7 +263,7 @@ open_with_row_selected (GtkCList *clist, gint row, gint column, GdkEventButton *
 }
 
 static void
-rename_file (GtkDialog *dialog_window, gint response_id)
+rename_file (GtkWidget *dialog_window, gint response_id)
 {
   gchar *dest, *error;
   GnomeVFSResult result;
@@ -347,6 +355,8 @@ popup_ask_move_file ()
   gtk_window_set_transient_for (GTK_WINDOW (dirbrowser_window), GTK_WINDOW (window));
 
   gtk_widget_show_all (dirbrowser_window);
+
+  refresh_current_directory();
 }
 
 static void
@@ -358,7 +368,7 @@ popup_ask_rename_file ()
 
   label_text = g_strdup_printf ("Rename file %s to:", current_popup_file->vfs->name);
 
-  dialog_window = gtk_dialog_new_with_buttons ("Rename file", window, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+  dialog_window = gtk_dialog_new_with_buttons ("Rename file", GTK_WINDOW (window), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
   g_signal_connect (G_OBJECT (dialog_window), "delete_event", GTK_SIGNAL_FUNC (kill_widget), dialog_window);
   g_signal_connect (G_OBJECT (dialog_window), "response", GTK_SIGNAL_FUNC (rename_file), NULL);
 
@@ -375,13 +385,13 @@ popup_ask_rename_file ()
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
 
   gtk_widget_show_all (dialog_window);
+
+  refresh_current_directory();
 }
 
 static void
 popup_ask_delete_file ()
 {
-  GtkWidget *dialog_window;
-  GtkWidget *label;
   gchar *label_text;
 
   label_text = g_strdup_printf (_("Delete \"%s\"?"), current_popup_file->vfs->name);
@@ -395,6 +405,8 @@ popup_ask_delete_file ()
 
       if (r != GNOME_VFS_OK)
 	gpe_error_box (gnome_vfs_result_to_string (r));
+      
+      refresh_current_directory();
     }
 
   g_free (label_text);
@@ -407,7 +419,7 @@ show_file_properties ()
 }
 
 static void
-ask_open_with (FileInfomation *file_info)
+ask_open_with (FileInformation *file_info)
 {
   GtkWidget *dialog_window, *fakeparentwindow, *clist, *entry, *label;
   GtkWidget *open_button, *cancel_button;
@@ -442,7 +454,7 @@ ask_open_with (FileInfomation *file_info)
                       entry);
 
   open_button = gpe_picture_button (dialog_window->style, "Open", "open");
-  gtk_object_set_data (GTK_OBJECT (open_button), "FileInfomation", (gpointer) file_info);
+  gtk_object_set_data (GTK_OBJECT (open_button), "FileInformation", (gpointer) file_info);
   gtk_signal_connect (GTK_OBJECT (open_button), "clicked",
                       GTK_SIGNAL_FUNC (open_with),
                       entry);
@@ -518,6 +530,12 @@ ask_open_with (FileInfomation *file_info)
   gtk_widget_show_all (dialog_window);	
 }
 
+static void
+send_with_bluetooth (void)
+{
+  bluetooth_send_file (current_popup_file->filename, current_popup_file->vfs);
+}
+
 static void
 popup_ask_open_with ()
 {
@@ -527,26 +545,56 @@ popup_ask_open_with ()
 void
 show_popup (GtkWidget *widget, gpointer udata)
 {
-  FileInfomation *file_info;
+  FileInformation *file_info;
 
-  file_info = (FileInfomation *) udata;
+  file_info = (FileInformation *) udata;
   current_popup_file = file_info;
 
-  printf ("popup for %s\n", file_info->filename);
+  if (bluetooth_available ())
+    gtk_widget_show (bluetooth_menu_item);
+  else
+    gtk_widget_hide (bluetooth_menu_item);
 
-  gtk_menu_popup (gtk_item_factory_get_widget (item_factory, "<main>"), NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time ());
+  gtk_menu_popup (GTK_MENU (gtk_item_factory_get_widget (item_factory, "<main>")), 
+		  NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time ());
 }
 
 void
 button_clicked (GtkWidget *widget, gpointer udata)
 {
   GnomeVFSMimeApplication *default_mime_application;
-  FileInfomation *file_info;
+  FileInformation *file_info;
   gchar *command;
 
-  file_info = (FileInfomation *) udata;
+  file_info = (FileInformation *) udata;
 
-  printf ("You clicked on %s\n", file_info->filename);
+  if (file_info == NULL)
+    return;
+
+  /* Handle Symbolic Links -- CM */
+  if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+    GnomeVFSURI *uri = gnome_vfs_uri_new (file_info->vfs->symlink_name);
+    GnomeVFSFileInfo *vfs = gnome_vfs_file_info_new ();
+    int cur_depth = MAX_SYM_DEPTH;
+
+    do {
+      uri = gnome_vfs_uri_new (file_info->vfs->symlink_name);
+      file_info->vfs = gnome_vfs_file_info_new ();
+
+      if (gnome_vfs_get_file_info_uri (uri, vfs, GNOME_VFS_FILE_INFO_DEFAULT) 
+          != GNOME_VFS_OK) {
+            printf ("Symbolic link %s leads to inaccessable file!\n", file_info->filename);
+            return;
+      }
+
+      file_info->vfs = vfs;
+    } while (file_info->vfs->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK && (--cur_depth > 0));
+
+    if (cur_depth <= 0) {
+      printf ("max symbolic link depth exceeded for %s\n", file_info->filename);
+      return;
+    }
+  }
 
   if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_REGULAR || file_info->vfs->type == GNOME_VFS_FILE_TYPE_UNKNOWN)
   {
@@ -569,9 +617,10 @@ button_clicked (GtkWidget *widget, gpointer udata)
   }
   else if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
     browse_directory (file_info->filename);
+  
 }
 
-gchar *
+const gchar *
 find_icon_path (gchar *mime_type)
 {
   struct stat s;
@@ -612,7 +661,7 @@ find_icon_path (gchar *mime_type)
 }
 
 GdkPixbuf *
-get_pixbuf (gchar *filename)
+get_pixbuf (const gchar *filename)
 {
   GdkPixbuf *pixbuf;
 
@@ -631,11 +680,11 @@ get_pixbuf (gchar *filename)
 }
 
 void
-add_icon (FileInfomation *file_info)
+add_icon (FileInformation *file_info)
 {
   GdkPixbuf *pixbuf;
-  gchar *mime_icon;
-
+  const gchar *mime_icon;
+  
   if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
     mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/directory.png");
   else if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_REGULAR || file_info->vfs->type == GNOME_VFS_FILE_TYPE_UNKNOWN)
@@ -646,6 +695,8 @@ add_icon (FileInfomation *file_info)
     else
       mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/regular.png");
   }
+  else if (file_info->vfs->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK)
+    mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/symlink.png");
   else
     mime_icon = g_strdup (PREFIX FILEMANAGER_ICON_PATH "/regular.png");
 
@@ -659,24 +710,27 @@ sort_filenames (gconstpointer *a, gconstpointer *b)
   return (strcoll ((char *) a, (char *) b));
 }
 
+gint
+compare_file_info (FileInformation *a, FileInformation *b)
+{
+  return strcoll (a->filename, b->filename);
+}
+
 /* Render the contents for the current directory. */
 static void
 make_view ()
 {
   GnomeVFSDirectoryHandle *handle;
   GnomeVFSFileInfo *vfs_file_info;
-  GnomeVFSURI *uri;
   GnomeVFSResult result, open_dir_result;
   loading_directory = 1;
+  GList *list = NULL, *iter;
 
   loaded_icons = g_hash_table_new (g_str_hash, g_str_equal);
-  gpe_iconlist_clear (view_widget);
+  gpe_iconlist_clear (GPE_ICONLIST (view_widget));
   gtk_widget_draw (view_widget, NULL);
 
-  uri = gnome_vfs_uri_new (current_directory);
-  printf ("GnomeVFS URI: %s\n", gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE));
-
-  open_dir_result = gnome_vfs_directory_open_from_uri (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
+  open_dir_result = gnome_vfs_directory_open (&handle, current_directory, GNOME_VFS_FILE_INFO_DEFAULT);
 
   while (open_dir_result == GNOME_VFS_OK)
   {
@@ -688,7 +742,7 @@ make_view ()
 
     if (vfs_file_info->name != NULL && vfs_file_info->name[0] != '.')
     {
-      FileInfomation *file_info = g_malloc (sizeof (*file_info));
+      FileInformation *file_info = g_malloc (sizeof (*file_info));
 	
       if (strcmp (current_directory, "/"))
         file_info->filename = g_strdup_printf ("%s/%s", current_directory, vfs_file_info->name);
@@ -697,12 +751,18 @@ make_view ()
 
       file_info->vfs = vfs_file_info;
 
-      add_icon (file_info);
+      list = g_list_insert_sorted (list, file_info, (GCompareFunc)compare_file_info);
     }
 
     if (result != GNOME_VFS_OK)
       break;
   }
+
+  for (iter = list; iter; iter = iter->next)
+    add_icon (iter->data);
+
+  g_list_free (iter);
+  
   gtk_widget_draw (view_widget, NULL);
   gnome_vfs_directory_close (handle);
   loading_directory = 0;
@@ -763,10 +823,18 @@ browse_directory (gchar *directory)
     //gpe_error_box ("No such file or directory.");
   }
 }
+
+static void
+refresh_current_directory (void)
+{
+  browse_directory (current_directory);
+}
+
+
 static void
 set_directory_home (GtkWidget *widget)
 {
-  browse_directory ("/mnt"); //(gchar *) g_get_home_dir ());
+  browse_directory ((gchar *) g_get_home_dir ());
 }
 
 static void
@@ -845,7 +913,7 @@ zoom_in ()
   {
     current_zoom = current_zoom + ZOOM_INCREMENT;
     printf ("ZOOMING IN!\n");
-    gpe_iconlist_set_icon_size (view_widget, current_zoom);
+    gpe_iconlist_set_icon_size (GPE_ICONLIST (view_widget), current_zoom);
   }
   printf ("Current zoom is %d\n", current_zoom);
 }
@@ -857,7 +925,7 @@ zoom_out ()
   {
     current_zoom = current_zoom - ZOOM_INCREMENT;
     printf ("ZOOMING OUT!\n");
-    gpe_iconlist_set_icon_size (view_widget, current_zoom);
+    gpe_iconlist_set_icon_size (GPE_ICONLIST (view_widget), current_zoom);
   }
   printf ("Current zoom is %d\n", current_zoom);
 }
@@ -881,10 +949,12 @@ main (int argc, char *argv[])
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
 
+  bluetooth_init ();
+
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (window), "Filemanager");
   gtk_widget_set_usize (GTK_WIDGET (window), window_x, window_y);
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
+  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
 		      GTK_SIGNAL_FUNC (gtk_exit), NULL);
 
   gtk_widget_realize (window);
@@ -901,7 +971,7 @@ main (int argc, char *argv[])
 		      GTK_SIGNAL_FUNC (button_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (view_widget), "show-popup",
 		      GTK_SIGNAL_FUNC (show_popup), NULL);
-  gpe_iconlist_set_icon_size (view_widget, current_zoom);
+  gpe_iconlist_set_icon_size (GPE_ICONLIST (view_widget), current_zoom);
 
   toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), GTK_ORIENTATION_HORIZONTAL);
@@ -946,7 +1016,8 @@ main (int argc, char *argv[])
   p = gpe_find_icon ("dir-up");
   pw = gpe_render_icon (window->style, p);
   gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Goto Location"), 
-			   _("Goto Location"), _("Goto Location"), pw, goto_directory, NULL);
+			   _("Goto Location"), _("Goto Location"), pw, 
+			   G_CALLBACK (goto_directory), NULL);
 
   gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (vbox));
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
@@ -963,6 +1034,8 @@ main (int argc, char *argv[])
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
 
+  bluetooth_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Send via Bluetooth");
+
   g_signal_connect (G_OBJECT (gtk_item_factory_get_widget (item_factory, "<main>")), "hide",
 		    GTK_SIGNAL_FUNC (hide_menu), NULL);
 
